using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Runtime.Serialization;
using System.Windows.Forms;
using System.IO;

using Syncfusion.Drawing;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;
using Syncfusion.Windows.Forms.Edit;

namespace EditControlSample
{
	public class EditCellModel : GridGenericControlCellModel
	{
		public EditCellModel(GridModel grid)
			: base(grid)
		{
		}
	
		protected EditCellModel(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
		}
		
		public override GridCellRendererBase CreateRenderer(GridControlBase control)
		{
			return new EditCellRenderer(control, this);
		}
	}
	
	public class EditCellRenderer: GridGenericControlCellRenderer
	{
		bool handleChanges;
		string changedValue;
		EditControl activeEditControl = new EditControl();
		EditControl staticEditControl = new EditControl();
		EditCellModel cm;
		
		public EditCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
			: base(grid, cellModel)
		{
			cm = (EditCellModel) cellModel;
			FixControlParent(activeEditControl);
			SetControl(activeEditControl);
			
			activeEditControl.Font = grid.Font;
			activeEditControl.StartWithNewFile = true;
			activeEditControl.ScrollBars = ScrollBars.None;
			activeEditControl.UserMarginVisible = false;
			activeEditControl.Multiline = false;
			activeEditControl.StatusBarVisible = false;
			activeEditControl.BorderStyle = BorderStyle.None;
			activeEditControl.IndicatorMarginVisible = false;
			activeEditControl.OutliningEnabled = false;
			activeEditControl.LineNumberMarginVisible = false;
			activeEditControl.AcceptsTab = false;
			activeEditControl.ShowSplitterButton = false;
			activeEditControl.SyntaxColoringEnabled = true;

			staticEditControl.Font = grid.Font;
			staticEditControl.StartWithNewFile = true;
			staticEditControl.ScrollBars = ScrollBars.None;
			staticEditControl.UserMarginVisible = false;
			staticEditControl.Multiline = false;
			staticEditControl.StatusBarVisible = false;
			staticEditControl.BorderStyle = BorderStyle.None;
			staticEditControl.IndicatorMarginVisible = false;
			staticEditControl.OutliningEnabled = false;
			staticEditControl.LineNumberMarginVisible = false;
			staticEditControl.AcceptsTab = false;
			staticEditControl.ShowSplitterButton = false;
			staticEditControl.SyntaxColoringEnabled = true;
			
			
			activeEditControl.TextChanged += new EventHandler(activeEditControl_TextChanged);

			this.handleChanges = false;
			this.changedValue = String.Empty;
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
				activeEditControl.TextChanged -= new EventHandler(activeEditControl_TextChanged);
				activeEditControl.Dispose();
				activeEditControl = null;
				staticEditControl.Dispose();
				staticEditControl = null;
				cm = null;
			}

			base.Dispose (disposing);
		}

		protected override bool OnSaveChanges()
		{
			if(CurrentCell.IsModified == true)
			{
				GridStyleInfo style = Grid.Model[this.RowIndex, this.ColIndex];
				style.CellValue = this.changedValue;
				
				if(base.NotifyCurrentCellChanging() == true)
				{
					ControlValue = this.changedValue;
					base.NotifyCurrentCellChanged();
				}
				
				return true;
			}
			return false;

		}

		protected override void OnInitialize(int rowIndex, int colIndex)
		{
			base.OnInitialize(rowIndex, colIndex);
			ControlValue = this.Grid.Model[rowIndex, colIndex].CellValue;
			FixControlParent(activeEditControl);
			if(this.ControlValue == null)
				this.ControlValue = "";
			this.activeEditControl.Text = ControlValue.ToString();
			this.Grid.CurrentCell.IsModified = false;
			
		}

		protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
		{

			Rectangle textRectangle = RemoveMargins(clientRectangle, style);

			if (CurrentCell.HasCurrentCellAt(rowIndex, colIndex) && 
				HasControlValue && ShouldDrawFocused(rowIndex, colIndex) && CurrentCell.IsEditing)
			{
				// Position current grid
				activeEditControl.Bounds = textRectangle;//clientRectangle;
				activeEditControl.BackColor = style.BackColor;
				if(!activeEditControl.Focused)
					activeEditControl.Focus();

			}
			else
			{
				this.handleChanges = false;
			
				// Determine the size for the edit control
				Rectangle controlRectangle = clientRectangle;
				controlRectangle.Inflate(-1, -1);
			
				// Set the value for the edit control
				string currentValue = (string)style.CellValue;
				staticEditControl.Text = currentValue;
			
				// Scroll to the beginning of the text value
				if(currentValue.Length > 0)
				{
					staticEditControl.CurrentChar = 1;
					staticEditControl.ScrollToViewCaret();
				}
			
				this.handleChanges = true;

				style.Control = staticEditControl;

				base.OnDraw (g, controlRectangle, rowIndex, colIndex, style);
			}
		}

		private void activeEditControl_TextChanged(object sender, EventArgs e)
		{
			if(this.handleChanges == true)
			{
				CurrentCell.IsModified = true;
				changedValue = activeEditControl.Text;
			}
		}
	}
}
